﻿using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.Visuals.RenderableSeries;
using SciChart.Data.Model;
using SciChart.Drawing.Common;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;

namespace SciChartMVVMTest
{
    public class CustomEnumRenderableSeriesViewModel : BaseRenderableSeriesViewModel
    {
        private float startY;
        public float StartY
        {
            get { return startY; }
            set {
                //startY = value;
                SetValue<float>(ref startY, value, "StartY");
            }
        }

        // Tell SciChart what type of RenderableSeries you want to instantiate
        public override Type RenderSeriesType
        {
            get { return typeof(CustomEnumRenderableSeries); }
        }
    }

    public sealed class CustomEnumRenderableSeries : CustomRenderableSeries
    {
        public CustomEnumRenderableSeries()
        {
            EnumerationValues = new List<string> { "test0", "test1", "test2", "test3" };
            ColorList = new List<Color> { Colors.Red, Colors.Green, Colors.Blue, Colors.Black };
            this.AntiAliasing = false;
        }

        public float StartY
        {
            get {
                return (float)GetValue(StartYProperty);
            }
            set {
                SetValue(StartYProperty, value);
            }
        }

        // Using a DependencyProperty as the backing store for StartY.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StartYProperty =
            DependencyProperty.Register("StartY", typeof(float), typeof(CustomEnumRenderableSeries), new PropertyMetadata(0f, (s,o) =>
            {
                //changed
                Console.Out.WriteLine("new value: " + o.NewValue);
            }));
        
        public float EndY
        {
            get { return (float)GetValue(EndYProperty); }
            set { SetValue(EndYProperty, value); }
        }

        // Using a DependencyProperty as the backing store for EndY.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EndYProperty =
            DependencyProperty.Register("EndY", typeof(float), typeof(CustomEnumRenderableSeries), new PropertyMetadata(1f));



        public float MarkerTipLength
        {
            get { return (float)GetValue(MarkerTipLengthProperty); }
            set { SetValue(MarkerTipLengthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for MarkerTipLength.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MarkerTipLengthProperty =
            DependencyProperty.Register("MarkerTipLength", typeof(float), typeof(CustomEnumRenderableSeries), new PropertyMetadata(4f));
        
        public float GapLength
        {
            get { return (float)GetValue(GapLengthProperty); }
            set { SetValue(GapLengthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for gapLength.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty GapLengthProperty =
            DependencyProperty.Register("gapLength", typeof(float), typeof(CustomEnumRenderableSeries), new PropertyMetadata(2f));
        
        public List<String> EnumerationValues
        {
            get { return (List<String>)GetValue(EnumerationValuesProperty); }
            set { SetValue(EnumerationValuesProperty, value); }
        }

        // Using a DependencyProperty as the backing store for EnumerationValues.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty EnumerationValuesProperty =
            DependencyProperty.Register("EnumerationValues", typeof(List<String>), typeof(CustomEnumRenderableSeries), new PropertyMetadata(null));
        
        public float Transparency
        {
            get { return (float)GetValue(TransparencyProperty); }
            set { SetValue(TransparencyProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Transparency.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty TransparencyProperty =
            DependencyProperty.Register("Transparency", typeof(float), typeof(CustomEnumRenderableSeries), new PropertyMetadata(0.05f));
        
        public List<IBrush2D> ColorBrushes;
        public List<Color> ColorList
        {
            get { return (List<Color>)GetValue(ColorListProperty); }
            set { SetValue(ColorListProperty, value); }
        }
        
        // Using a DependencyProperty as the backing store for colors.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ColorListProperty =
            DependencyProperty.Register("colors", typeof(List<Color>), typeof(CustomEnumRenderableSeries), new PropertyMetadata(null,
                (o,e) => {
                    (o as CustomEnumRenderableSeries).ColorBrushes = null;
                }));
        
        private IBrush2D defaultBrush;
        private IPen2D pen;
        
        protected override void Draw(IRenderContext2D renderContext, IRenderPassData renderPassData)
        {
            IPointSeries pointSeries = renderPassData.PointSeries as IPointSeries;

            //update our color brushes
            if (ColorList != null && ColorBrushes == null)
            {
                ColorBrushes = new List<IBrush2D>();
                foreach (Color color in ColorList)
                {
                    ColorBrushes.Add(renderContext.CreateBrush(color, Transparency));
                }
            }

            int setCount = pointSeries.Count;

            if (pen == null)
            {
                defaultBrush = renderContext.CreateBrush(Brushes.Black, 0.05f);
                pen = renderContext.CreatePen(Colors.Black, false, 1);
            }
            
            // Iterate over points collection and render point markers
            for (int i = 0; i < setCount; i++)
            {
                double xPoint = (double)pointSeries.XValues[i];//pointSeries[i].X;
                int yPoint = (int)pointSeries.YValues[i];//pointSeries[i].Y;
                
                // Get coordinates for X,Y data values
                var x1 = (int)renderPassData.XCoordinateCalculator.GetCoordinate(xPoint);
                var y1 = 1 + (int)(StartY * renderPassData.TransformationStrategy.ViewportSize.Height);
                var y2 = (int)(EndY * renderPassData.TransformationStrategy.ViewportSize.Height);

                string valueText = "";
                if (yPoint >= 0 && yPoint < EnumerationValues.Count)
                {
                    valueText = EnumerationValues[yPoint];
                }
                IBrush2D brush = defaultBrush;
                if (ColorBrushes != null && yPoint >= 0 && yPoint < ColorBrushes.Count)
                {
                    brush = ColorBrushes[yPoint];
                }

                //var pointMarkerRect = new Rect(0, 0, pointMarker.PixelWidth, pointMarker.PixelWidth);
                //double xOffset = pointMarkerRect.Width / 2;
                //double yOffset = pointMarkerRect.Height / 2;



                int x2 = 0;
                if (i == setCount-1)
                {
                    x2 = (int)renderPassData.TransformationStrategy.ViewportSize.Width;
                }
                else
                {
                    double x2Point = (double)pointSeries.XValues[i + 1];
                    x2 = (int)renderPassData.XCoordinateCalculator.GetCoordinate(x2Point);
                }

                bool rightLimit = false;
                if (x2 == renderPassData.TransformationStrategy.ViewportSize.Width)
                {
                    rightLimit = true;
                }

                // Draw PointMarkers
                renderContext.FillRectangle(brush, new Point(x1 + GapLength, y1), new Point(x2 - GapLength, y2));
                renderContext.DrawText(new Rect(Math.Max(x1,0), y1, Math.Max(0,x2 - Math.Max(x1, 0)), Math.Max(0, y2 - y1)), Colors.Black, (float)FontSize, valueText);

                
                renderContext.DrawLine(pen, new Point(x1 + GapLength, y1), new Point(x1 + GapLength, y2));
                if (!rightLimit)
                {
                    renderContext.DrawLine(pen, new Point(x2 - GapLength, y1), new Point(x2 - GapLength, y2));
                }
                if (x2-x1 > 2* MarkerTipLength)
                {
                    renderContext.DrawLine(pen, new Point(x1 + GapLength, y1), new Point(x1 + GapLength + MarkerTipLength, y1));
                    renderContext.DrawLine(pen, new Point(x1 + GapLength, y2), new Point(x1 + GapLength + MarkerTipLength, y2));

                    if (!rightLimit)
                    {
                        renderContext.DrawLine(pen, new Point(x2 - GapLength, y1), new Point(x2 - GapLength - MarkerTipLength, y1));
                        renderContext.DrawLine(pen, new Point(x2 - GapLength, y2), new Point(x2 - GapLength - MarkerTipLength, y2));
                    }
                }
                else
                {
                    renderContext.DrawLine(pen, new Point(x1 +GapLength, y1), new Point(x2 -GapLength, y1));
                    renderContext.DrawLine(pen, new Point(x1 +GapLength, y2), new Point(x2 -GapLength, y2));
                }
                //renderContext.DrawEllipse(pen, brush, new Point(x1, y1), z1, z1); //(new Rect(x1 - xOffset, y1 - yOffset, pointMarkerRect.Width, pointMarkerRect.Height), pointMarker, pointMarkerRect);
            }
        }
    }
}
